import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.*;

public class TTTModelTest {

    private TTTModel TTTGrid;

    @Before
    public void instantiateGrid(){
        TTTGrid = new TTTModel();
    }

    @Test
    // testing if correct TTT board is being built
    // assumption: when a new board object is being built it is size 3x3
    public void testBoardConstructor1(){
        assertEquals(3,TTTGrid.getGrid().length);
        assertFalse( TTTGrid.getGrid().length > 3 && TTTGrid.getGrid().length < 3); // should not be any other size
    }

    @Test
    // testing what symbol is on each grid tile after building new object
    // assumption: all tiles should initially be empty when new board is created

    public void testBoardConstructor2(){

        for (int i = 0; i < 3; i++) {
            for (int j = 0; j < 3; j++) {
                assertEquals(Symbol.Empty,TTTGrid.getTile(i,j));
            }
        }

    }

    @Test
    // testing mark tile method for grid
    // assumption: when tile is marked it should not be empty afterwards but either have X or O
    public void testMarkTile(){

        TTTGrid.markTile(Symbol.X, 2, 0);
        assertNotEquals(Symbol.Empty, TTTGrid.getTile(2,0));
        assertEquals(Symbol.X, TTTGrid.getTile(2,0));

        TTTGrid.markTile(Symbol.O, 0, 1);
        assertNotEquals(Symbol.Empty, TTTGrid.getTile(0,1));
        assertEquals(Symbol.O, TTTGrid.getTile(0,1));
    }


    @Test
    // testing if three X/O row, column or vertical are the winning state
    // assumption: if three X -> X should win, if three O -> O should win, if not three then game is ongoing

    public void testBoardStateXWinsColumn(){

        TTTGrid.markTile(Symbol.X, 0, 0);
        TTTGrid.markTile(Symbol.X, 1, 0);
        TTTGrid.markTile(Symbol.X, 2, 0);
        assertEquals(BoardState.XWin,TTTGrid.checkState());
    }

    @Test
    public void testBoardStateOWinsRow(){

        TTTGrid.markTile(Symbol.O, 0, 0);
        TTTGrid.markTile(Symbol.O, 0, 1);
        TTTGrid.markTile(Symbol.O, 0, 2);
        assertEquals(BoardState.OWin,TTTGrid.checkState());
    }

    @Test
    public void testBoardStateOnGoingVertical(){

        TTTGrid.markTile(Symbol.O, 0, 0);
        TTTGrid.markTile(Symbol.X, 1, 1);
        TTTGrid.markTile(Symbol.O, 2, 2);
        assertEquals(BoardState.Ongoing,TTTGrid.checkState());
    }

    @Test
    // testing the toString method of the TTTModel class
    // assumption: all tile symbols should be shown in string exactly as 'placed' in the grid

    public void testToStringOfGrid(){

        TTTGrid.markTile(Symbol.X, 1, 0);
        TTTGrid.markTile(Symbol.X, 1, 1);
        TTTGrid.markTile(Symbol.O, 0, 0);
        TTTGrid.markTile(Symbol.O, 0, 2);
        assertEquals("O| |O|" + "______" + "X|X| |" + "______" + "| | |" + "______", TTTGrid.toString());
    }

}